# yubico-web-client-lib.pl
# Function used to access web services 
BEGIN { push(@INC, ".."); };
use WebminCore;
use LWP::Simple;
use URI::Escape;
use Digest::MD5 qw(md5_hex);
&init_config();

sub yubikey_webclient
{
  my ($service_url,$otp,$id,$api) = @_;

  my $params = "id=$id&otp=" . uri_escape($otp) ;
  $params .= '&nonce=' . uri_escape(md5_hex(rand()));
  if (defined $api) {
    $params .= '&h=' . uri_escape(encode_base64(hmac_sha1($params, decode_base64($api)), ''));
  } 

  my $response = get("$service_url?$params");
  &webmin_debug_log("otp_validate", "otp url". "$service_url?$params");
  &webmin_debug_log("otp_validate", "otp response $response");

  chomp($response);
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }

  if (defined $api) {
    #extract each of the lines, and store in a hash...

    my %result;	
    foreach (split(/\n/,$response))
    {
      chomp;
      if($_ =~ /=/)
      {
	($a,$b) = split(/=/,$_,2);
	$b =~ s/\s//g;
	$result{$a} = $b;
      }
    }

    # save the h parameter, that's what we'll be comparing to

    my $signatur=$result{h};
    delete $result{h};
    my $datastring='';

    my $key;
    foreach $key (sort keys %result)
    {
      $result{$key} =~ s/\s//g;
      $datastring .= "$key=$result{$key}&";
    }
    $datastring = substr($datastring,0,length($datastring)-1);

    my $hmac = encode_base64(hmac_sha1($datastring,decode_base64($api)));

    chomp($hmac);

    if($hmac eq $signatur)
    {
      return "OK";
    }
    else
    {
      return "ERR_HMAC";
    } 
  }
  else 
  {
    return "OK";
  }
}


sub ykmap_add_mapping
{
  my ($service_url,$username,$yubikey_id) = @_;

  my $params = "find=yubikey_prefix&record=" . uri_escape($yubikey_id) . "&keyword=username&value=" . uri_escape(lc($username)) ;

  my $response = get($service_url . "map-store?" . $params);

  chomp($response);
  if($response !~ /status=ok/i)
  {
    &webmin_debug_log("ykmap_add_mapping:", $service_url . "map-store?" . $params);
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}


sub ykmap_get_yubikeyid_mapping
{
  my ($service_url,$username) = @_;

  my $params = "find=username&record=" . uri_escape(lc($username)) . "&keyword=yubikey_prefix" ;

  my $response = get($service_url . "map-query?" . $params);
  my %result = {}; 

  chomp($response);
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    &webmin_debug_log("ykmap_get_user_mapping:", "params:  $service_url -  $params response: $response ");
    $response =~ m/status=(.+)\r/;
    $result{'status'} = "ERR_$1";
    return \%result;
  }

  #extract each of the lines, and store in a hash...
  local (@data);
  foreach (split(/\r\n/,$response))
  {
    chomp;
    if($_ =~ /yubikey_prefix/ ) 
    {
      local %row;
      ($a,$b) = split(/ /,$_,2);
      ($k,$v) = split(/=/,$a,2);
      $row{$k} = $v;
      ($k,$v) = split(/=/,$b,2);
      $row{$k} = $v;
      push(@data, { %row });

    }
  }

  $result{'status'} = "OK";
  $result{'data'} = \@data;
  return \%result;
}

sub ykmap_get_users_mapping
{
  my ($service_url,$yubikeyid) = @_;
  my $params = "find=yubikey_prefix&record=" . uri_escape($yubikeyid) . "&keyword=username" ;
  my $response = get($service_url . "map-query?" . $params);
  my %result = {}; 

  chomp($response);
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    &webmin_debug_log("ykmap_get_users_mapping:", "params:  $params response: $response");
    $response =~ m/status=(.+)\r/;
    $result{'status'} = "ERR_$1";
    return \%result;
  }

  #extract each of the lines, and store in a hash...
  local (@data);
  foreach (split(/\r\n/,$response))
  {
    chomp;
    if($_ =~ /username/ ) 
    {
      local %row;
      ($a,$b) = split(/ /,$_,2);
      ($k,$v) = split(/=/,$a,2);
      $row{$k} = $v;
      ($k,$v) = split(/=/,$b,2);
      $row{$k} = $v;
      push(@data, { %row });

    }
  }

  $result{'status'} = "OK";
  $result{'data'} = \@data;

  #print %result;
  
  #&ui_print_header(undef, $text{'index_title'}, "", undef, 1, 1);
  #$dumper = Dumpvalue->new();
  #$dumper->set(compactDump =>false);
  #$dump = $dumper->dumpValue(\%result);
  
  #&webmin_debug_log("ykmap_get_users_mapping:", "result ===>"." $result{'username'}");
  return \%result;
}

sub ykmap_update_mapping
{
  my ($service_url,$username,$yubikey_id,$status) = @_;

  my $params = "find=yubikey_prefix&record=" . uri_escape($yubikey_id) . "&keyword=username&value=" . uri_escape(lc($username)) .
  "&status=" . uri_escape($status);

  my $response = get($service_url . "map-update?" . $params);

  chomp($response);
  &webmin_debug_log("ykmap_update_mapping:", "params:  $params response: $response");
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}
sub ykmap_update_rename
{
  my ($service_url,$old_username,$new_username) = @_;
  my $params = "find=yubikey_prefix&old_username=" . uri_escape($old_username) . "&keyword=username&new_username=" . uri_escape($new_username);
  my $response = get($service_url . "map-rename?" . $params);
  chomp($response);
  &webmin_debug_log("ykmap_update_mapping:", "params:  $params response: $response");
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}
sub ykmap_get_mappings
{
  my ($service_url,$filter) = @_;

  my $params = "find=yubikey_report&value=" . uri_escape($filter);

  my $response = get($service_url . "map-query?" . $params);

  my $report_content = "";

  chomp($response);
  if($response !~ /status=ok/i)
  {
    # If the status is not ok, let's not even go through the rest...
    &webmin_debug_log("ykmap_get_mappings:", "params:  $params response: $response");
    return $report_content;
  }

  #extract each of the lines, and store in a hash...
  local (@data);
  foreach (split(/\r\n/,$response))
  {
    chomp;
    if($_ =~ /report_content/ )
    {
      $report_content = uri_unescape($_);
    }
  }

  return $report_content;
}

sub ykmap_delete_mapping
{
  my ($service_url,$username,$yubikey_id) = @_;

  my $params = "find=yubikey_prefix&record=" . uri_escape($yubikey_id) . "&keyword=username&value=" . uri_escape(lc($username)) ;
  
  my $response = get($service_url . "map-delete?" . $params);

  chomp($response);
  if($response !~ /status=ok/i)
  {
    &webmin_debug_log("ykmap_delete_mapping:", $service_url . "map-delete?" . $params);
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}

sub ykmap_delete_user_all_mappings
{
  my ($service_url,$username,$yubikey_id) = @_;

  my $params = "find=yubikey_prefix&record=" . uri_escape($yubikey_id) . "&keyword=username&value=" . uri_escape(lc($username)) . "&flag=delete_user";

  my $response = get($service_url . "map-delete?" . $params);

  chomp($response);
  if($response !~ /status=ok/i)
  {
    &webmin_debug_log("ykmap_delete_mapping:", $service_url . "map-delete?" . $params);
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}

sub ykmap_delete_user_mappings_on_domain_delete
{
  my ($service_url, $domain_name) = @_;

  my $params = "find=domain_name&record=" . uri_escape($domain_name) . "&flag=delete_all_domain_user";

  my $response = get($service_url . "map-delete?" . $params);

  chomp($response);
  if($response !~ /status=ok/i)
  {
    &webmin_debug_log("ykmap_delete_mapping_on_domain_delete:", $service_url . "map-delete?" . $params);
    # If the status is not ok, let's not even go through the rest...
    $response =~ m/status=(.+)/;
    return "ERR_$1";
  }
  return "OK";
}


1;
